@@ -1,4 +1,28 @@
+/*
+ * The MIT License (MIT)
+ *
+ * Copyright (c) 2016-2023 Intel Corporation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
+ */
 #include<stdio.h>
+#include "smithwaterman_common.h"
 
 #define MAIN_CODE(bt_vec) \
             { \
@@ -36,12 +60,13 @@
             VEC_STOREU((VEC_INT_TYPE *)(&H[hCurInd]), h11); \
             }
 
+static uint32_t D_MAX_SEQ_LEN = MAX_SEQ_LEN;
 
 void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch, int32_t open, int32_t extend, int32_t* E_,int32_t tid)
 {
-    uint32_t seq1[MAX_SEQ_LEN];
-    uint32_t seq1Rev[MAX_SEQ_LEN];
-    uint32_t seq2[MAX_SEQ_LEN];
+    uint32_t seq1[D_MAX_SEQ_LEN];
+    uint32_t seq1Rev[D_MAX_SEQ_LEN];
+    uint32_t seq2[D_MAX_SEQ_LEN];
     int32_t nrow = p->len1;
     int32_t ncol = p->len2;
     int32_t overhangStrategy = p->overhangStrategy;
@@ -61,10 +86,10 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
     VEC_INT_TYPE ins_ext_vec = VEC_SET1_VAL32(INSERT_EXT);
     VEC_INT_TYPE del_ext_vec = VEC_SET1_VAL32(DELETE_EXT);
     INIT_CONSTANTS;
-    int32_t hwidth = MAX_SEQ_LEN + AVX_LENGTH;
-    int32_t ewidth = MAX_SEQ_LEN + AVX_LENGTH;
+    int32_t hwidth = D_MAX_SEQ_LEN + AVX_LENGTH;
+    int32_t ewidth = D_MAX_SEQ_LEN + AVX_LENGTH;
 
-    int32_t *E  = E_ + tid * 6 * (MAX_SEQ_LEN + AVX_LENGTH);
+    int32_t *E  = E_ + tid * 6 * (D_MAX_SEQ_LEN + AVX_LENGTH);
     int32_t *F  = E + 1 * ewidth;
     int32_t *H  = E + 2 * ewidth;
     for(j = 0; j <= ncol; j+=AVX_LENGTH)
@@ -76,12 +101,12 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
         VEC_STORE(E + i, lowInitValue_vec);
     }
 
-    H[MAX_SEQ_LEN >> 1] = 0;
+    H[D_MAX_SEQ_LEN >> 1] = 0;
 
     for(i = 0; i < nrow; i++)
     {
         seq1[i] = p->seq1[i];
-        seq1Rev[MAX_SEQ_LEN - 1 - i] = p->seq1[i];
+        seq1Rev[D_MAX_SEQ_LEN - 1 - i] = p->seq1[i];
     }
     for(i = 0; i < ncol; i++)
     {
@@ -124,8 +149,8 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
             {
             i = antiDiag - j;
             int32_t diag = j - i;
-            int32_t diagInd = MAX_SEQ_LEN + diag;
-            int32_t inde = MAX_SEQ_LEN - i;
+            int32_t diagInd = D_MAX_SEQ_LEN + diag;
+            int32_t inde = D_MAX_SEQ_LEN - i;
             int32_t indf = j;
             int32_t hLeftInd = prev + ((diagInd - 1) >> 1);
             int32_t hTopInd  = hLeftInd + 1;
@@ -137,8 +162,8 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
             {
             i = antiDiag - j;
             int32_t diag = j - i;
-            int32_t diagInd = MAX_SEQ_LEN + diag;
-            int32_t inde = MAX_SEQ_LEN - i;
+            int32_t diagInd = D_MAX_SEQ_LEN + diag;
+            int32_t inde = D_MAX_SEQ_LEN - i;
             int32_t indf = j;
             int32_t hLeftInd = prev + ((diagInd - 1) >> 1);
             int32_t hTopInd  = hLeftInd + 1;
@@ -151,7 +176,7 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
             VEC_INT_TYPE bt_vec_2 = VEC_PERMUTE2x128_EVEN(bt_vec_0, bt_vec_1);
             VEC_INT_TYPE bt_vec_3 = VEC_PERMUTE2x128_ODD(bt_vec_0, bt_vec_1);
             VEC_INT_TYPE bt_vec   = VEC_PACKS_32(bt_vec_2, bt_vec_3);
-            VEC_STREAM(backTrack + antiDiag * MAX_SEQ_LEN + backTrackInd, bt_vec);
+            VEC_STREAM(backTrack + antiDiag * D_MAX_SEQ_LEN + backTrackInd, bt_vec);
         }
 #ifdef IACA_ANALYSIS
         IACA_END
@@ -163,8 +188,8 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
 #endif
             i = antiDiag - j;
             int32_t diag = j - i;
-            int32_t diagInd = MAX_SEQ_LEN + diag;
-            int32_t inde = MAX_SEQ_LEN - i;
+            int32_t diagInd = D_MAX_SEQ_LEN + diag;
+            int32_t inde = D_MAX_SEQ_LEN - i;
             int32_t indf = j;
             int32_t hLeftInd = prev + ((diagInd - 1) >> 1);
             int32_t hTopInd  = hLeftInd + 1;
@@ -177,7 +202,7 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
             VEC_INT_TYPE bt_vec_3 = VEC_PERMUTE2x128_ODD(bt_vec_0, bt_vec_1);
             VEC_INT_TYPE bt_vec   = VEC_PACKS_32(bt_vec_2, bt_vec_3);
             int32_t backTrackInd = j - jlo - 1;
-            VEC_STREAM(backTrack + antiDiag * MAX_SEQ_LEN + backTrackInd, bt_vec);
+            VEC_STREAM(backTrack + antiDiag * D_MAX_SEQ_LEN + backTrackInd, bt_vec);
         }
 #ifdef PERF_DEBUG
         endTick = __rdtsc();
@@ -186,21 +211,21 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
 #endif
         if((overhangStrategy == INDEL) || (overhangStrategy == LEADING_INDEL))
         {
-            H[cur + ((MAX_SEQ_LEN + 2 * jhi - antiDiag) >> 1)] = open + (jhi - 1) * extend;
-            H[cur + ((MAX_SEQ_LEN + 2 * jlo - antiDiag) >> 1)] = open + (ilo - 1) * extend;
+            H[cur + ((D_MAX_SEQ_LEN + 2 * jhi - antiDiag) >> 1)] = open + (jhi - 1) * extend;
+            H[cur + ((D_MAX_SEQ_LEN + 2 * jlo - antiDiag) >> 1)] = open + (ilo - 1) * extend;
         }
         else
         {
-            H[cur + ((MAX_SEQ_LEN + 2 * jhi - antiDiag) >> 1)] = 0;
-            H[cur + ((MAX_SEQ_LEN + 2 * jlo - antiDiag) >> 1)] = 0;
+            H[cur + ((D_MAX_SEQ_LEN + 2 * jhi - antiDiag) >> 1)] = 0;
+            H[cur + ((D_MAX_SEQ_LEN + 2 * jlo - antiDiag) >> 1)] = 0;
         }
         F[jhi] = lowInitValue;
-        E[MAX_SEQ_LEN - ilo] = lowInitValue;
+        E[D_MAX_SEQ_LEN - ilo] = lowInitValue;
 
 #if 1
         if(ilo == (nrow + 1))
         {
-            int32_t score = H[cur + ((MAX_SEQ_LEN + jlo + 1 - (ilo - 1)) >> 1)];
+            int32_t score = H[cur + ((D_MAX_SEQ_LEN + jlo + 1 - (ilo - 1)) >> 1)];
             if((overhangStrategy == SOFTCLIP) || (overhangStrategy == IGNORE))
             {
                 if((maxScore < score) || ((maxScore == score) && (abs(ilo - jlo - 2) < abs(max_i - max_j))))
@@ -214,7 +239,7 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
         }
         if(jhi == (ncol + 1))
         {
-            int32_t score = H[cur + ((MAX_SEQ_LEN + jhi - 1 - (ihi + 1)) >> 1)];
+            int32_t score = H[cur + ((D_MAX_SEQ_LEN + jhi - 1 - (ihi + 1)) >> 1)];
             if((maxScore < score) || ((maxScore == score) && ((max_j == ncol) || (abs(ihi - jhi + 2) <= abs(max_i - max_j)))))
             {
                 //printf("%d, %d) hij = %d\n", i, j, h11);
@@ -227,25 +252,26 @@ void inline smithWatermanBackTrack(SeqPair *p, int32_t match, int32_t mismatch,
     }
     //printf("maxScore = %d, max_i = %d, max_j = %d\n", maxScore, max_i, max_j);
     if(overhangStrategy == INDEL)
-        p->score = H[cur + ((MAX_SEQ_LEN + ncol - nrow) >> 1)];
+        p->score = H[cur + ((D_MAX_SEQ_LEN + ncol - nrow) >> 1)];
     else
         p->score = maxScore;
     p->score = maxScore;
-    p->max_i = max_i;
-    p->max_j = max_j;
+    // i and j are bounded by the length of the input which is at most 16 bit.
+    p->max_i = (int16_t)max_i;
+    p->max_j = (int16_t)max_j;
     return;
 }
 
-void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
+void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t cigarBufLength, int32_t tid)
 {
     int16_t *btrack = p->btrack;
-    int32_t max_i = p->max_i;
-    int32_t max_j = p->max_j;
-    int32_t nrow = p->len1;
-    int32_t ncol = p->len2;
-    int32_t overhangStrategy = p->overhangStrategy;
+    int16_t max_i = p->max_i;
+    int16_t max_j = p->max_j;
+    int16_t nrow = p->len1;
+    int16_t ncol = p->len2;
+    int8_t overhangStrategy = p->overhangStrategy;
 
-    int32_t i, j;
+    int16_t i, j;
     int16_t *cigarArray = cigarBuf_;
 
     int32_t cigarId = 0;
@@ -269,7 +295,9 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
     if(j < ncol)
     {
         cigarArray[cigarId * 2] = SOFTCLIP;
-        cigarArray[cigarId * 2 + 1] = ncol - j;
+        // ncol - j is automatically converted to an int because the result of int16_t - int16_t may overflow int16_t
+        // That never happens in this case as both are guaranteed to be positive and j < ncol.
+        cigarArray[cigarId * 2 + 1] = (int16_t)(ncol - j);
         cigarId++;
     }
     int state = 0;
@@ -279,12 +307,12 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
         int32_t btrInd;
         if(antiDiag <= nrow)
         {
-            btrInd = antiDiag * MAX_SEQ_LEN + j - 1;
+            btrInd = antiDiag * D_MAX_SEQ_LEN + j - 1;
         }
         else
         {
             int32_t jLo = antiDiag - nrow - 1;
-            btrInd = antiDiag * MAX_SEQ_LEN + j - jLo - 1;
+            btrInd = antiDiag * D_MAX_SEQ_LEN + j - jLo - 1;
         }
         int32_t btr = btrack[btrInd];
         if(state == INSERT_EXT)
@@ -331,7 +359,7 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
     if(overhangStrategy == SOFTCLIP)
     {
         if(j > 0)
-        {
+        {   
             cigarArray[cigarId * 2] = SOFTCLIP;
             cigarArray[cigarId * 2 + 1] = j;
             cigarId++;
@@ -346,10 +374,12 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
             cigarArray[cigarId * 2 + 1] = j;
             cigarId++;
         }
-        p->alignmentOffset = i - j;
+        // i - j is automatically converted to an int because the result of int16_t - int16_t may overflow int16_t
+        // That never happens in this case as both are guaranteed to be positive.
+        p->alignmentOffset = (int16_t)(i - j);
     }
     else // overhangStrategy == INDEL || overhangStrategy == LEADING_INDEL
-    {
+    {   
         if (i > 0)
         {
             cigarArray[cigarId * 2] = DELETE;
@@ -364,15 +394,16 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
         }
         p->alignmentOffset = 0;
     }
-    int32_t newId = 0;
-    int32_t prev = cigarArray[newId * 2];
+    int16_t newId = 0;
+    int16_t prev = cigarArray[newId * 2];
     //printf("cigarId = %d\n", cigarId);
     for(i = 1; i < cigarId; i++)
     {
-        int32_t cur = cigarArray[i * 2];
+        int16_t cur = cigarArray[i * 2];
         if(cur == prev)
         {
-            cigarArray[newId * 2 + 1] += cigarArray[i * 2 + 1];
+            // cigarArray elements will never overflow int16_t because they're bounded by the 16 bit input length.
+            cigarArray[newId * 2 + 1] = (int16_t)(cigarArray[newId * 2 + 1] + cigarArray[i * 2 + 1]);
         }
         else
         {
@@ -387,69 +418,84 @@ void inline getCIGAR(SeqPair *p, int16_t *cigarBuf_, int32_t tid)
     int curSize = 0;
     for(i = newId; i >= 0; i--)
     {
-        int bytesWritten = sprintf(p->cigar + curSize, "%d", cigarArray[2 * i + 1]);
-        curSize += bytesWritten;
+        char state;
 
         switch(cigarArray[2 * i])
         {
             case MATCH:
-                sprintf(p->cigar + curSize, "M");
+                state = 'M';
                 break;
             case INSERT:
-                sprintf(p->cigar + curSize, "I");
+                state = 'I';
                 break;
             case DELETE:
-                sprintf(p->cigar + curSize, "D");
+                state = 'D';
                 break;
             case SOFTCLIP:
-                sprintf(p->cigar + curSize, "S");
+                state = 'S';
                 break;
             default:
-                sprintf(p->cigar + curSize, "R");
+                state = 'R';
                 break;
         }
-        curSize++;
+        int expectedLength = fast_itoa(NULL, cigarArray[2 * i + 1]) + 1;
+
+        if (curSize >= 0 && expectedLength > 1 && curSize + expectedLength <= cigarBufLength){ 
+            curSize += fast_itoa(p->cigar + curSize, cigarArray[2 * i + 1]);
+            p->cigar[curSize++] = state;
+
+        }
     }
-    p->cigarCount = strlen(p->cigar);
+    // CIGARs are specified to have their length fit in uint32_t
+    p->cigarCount = (uint32_t)strnlen(p->cigar, curSize);
 }
 
 
-int32_t CONCAT(runSWOnePairBT_,SIMD_ENGINE)(int32_t match, int32_t mismatch, int32_t open, int32_t extend,uint8_t *seq1, uint8_t *seq2, int32_t len1, int32_t len2, int8_t overhangStrategy, char *cigarArray, int16_t *cigarCount)
+int32_t CONCAT(runSWOnePairBT_,SIMD_ENGINE)(int32_t match, int32_t mismatch, int32_t open, int32_t extend,uint8_t *seq1, uint8_t *seq2, int16_t len1, int16_t len2, int8_t overhangStrategy, char *cigarArray, int32_t cigarLen, uint32_t *cigarCount, int32_t *offset)
 {
-
-
-
     int32_t  w_match = match;
     int32_t  w_mismatch = mismatch;
     int32_t  w_open = open;
     int32_t  w_extend = extend;
 
-    int32_t  *E_  = (int32_t *)_mm_malloc((6 * (MAX_SEQ_LEN+ AVX_LENGTH)) * sizeof(int32_t), 64);
-    int16_t  *backTrack_ = (int16_t *)_mm_malloc((2 * MAX_SEQ_LEN * MAX_SEQ_LEN + 2 * AVX_LENGTH) * sizeof(int16_t), 64);
-    int16_t  *cigarBuf_  = (int16_t *)_mm_malloc(4 * MAX_SEQ_LEN * sizeof(int16_t), 64);
-   
-    if (E_ == NULL  | backTrack_  == NULL | cigarBuf_ == NULL) {
+    
+    D_MAX_SEQ_LEN = MAX_SEQ_LEN;
+
+    int len = max(len1, len2);
+
+    while(len >= D_MAX_SEQ_LEN){
+	        D_MAX_SEQ_LEN += MAX_SEQ_LEN;
+    }
+
+    int32_t  *E_  = (int32_t *)_mm_malloc((6 * (D_MAX_SEQ_LEN+ AVX_LENGTH)) * sizeof(int32_t), 64);
+    int16_t  *backTrack_ = (int16_t *)_mm_malloc((2 * D_MAX_SEQ_LEN * D_MAX_SEQ_LEN + 2 * AVX_LENGTH) * sizeof(int16_t), 64);
+    int16_t  *cigarBuf_  = (int16_t *)_mm_malloc(4 * D_MAX_SEQ_LEN * sizeof(int16_t), 64);
+
+    if (E_ == NULL  || backTrack_  == NULL || cigarBuf_ == NULL) {
          _mm_free(E_);
          _mm_free(backTrack_);
          _mm_free(cigarBuf_);
-	 return -1; 
+	 return SW_MEMORY_ALLOCATION_FAILED;
     } 
 
     SeqPair p;
     p.seq1 = seq1;
     p.seq2 = seq2;
+    // len1 and len2 are bounded by the length of the input with is at most 16 bit.
     p.len1 = len1;
     p.len2 = len2;
     p.overhangStrategy = overhangStrategy;
     p.btrack = backTrack_;
     p.cigar = cigarArray;
-    smithWatermanBackTrack(&p, match, mismatch,  open, extend, E_, 0);
-    getCIGAR(&p, cigarBuf_, 0);
 
+    smithWatermanBackTrack(&p, match, mismatch,  open, extend, E_, 0);
+    getCIGAR(&p, cigarBuf_, cigarLen, 0);
     (*cigarCount) = p.cigarCount;
 
     _mm_free(E_);
     _mm_free(backTrack_);
     _mm_free(cigarBuf_);
-    return p.alignmentOffset;
+
+    *offset = p.alignmentOffset;
+    return SW_SUCCESS;
     }
